from typing import Optional

import typer

from service_sdk.client import Client
from service_sdk.service import DEFAULT_SERVICE_HOST, DEFAULT_SERVICE_PORT

cli = typer.Typer()


@cli.command()
def ping(host: str = DEFAULT_SERVICE_HOST, port: str = DEFAULT_SERVICE_PORT):
    result = Client(host=host, port=port).ping()
    print(result)


@cli.command("exit")
def exit_service(
        host: str = DEFAULT_SERVICE_HOST, port: str = DEFAULT_SERVICE_PORT, force: Optional[bool] = None, wait: Optional[bool] = None
):
    result = Client(host=host, port=port).exit(force=force, wait=wait)
    print(result)


@cli.command()
def stop_all(
        host: str = DEFAULT_SERVICE_HOST, port: str = DEFAULT_SERVICE_PORT, force: Optional[bool] = None, wait: Optional[bool] = None
):
    Client(host=host, port=port).stop_all(force=force, wait=wait)


@cli.command()
def kill(host: str = DEFAULT_SERVICE_HOST, port: str = DEFAULT_SERVICE_PORT):
    result = Client(host=host, port=port).kill_service()
    print(result)


@cli.command()
def start(host: str = DEFAULT_SERVICE_HOST, port: str = DEFAULT_SERVICE_PORT):
    result = Client(host=host, port=port).start_worker()
    print(result)


@cli.command()
def stop(uid: str, host: str = DEFAULT_SERVICE_HOST, port: str = DEFAULT_SERVICE_PORT):
    result = Client(host=host, port=port).stop_worker(uid=uid)
    print(result)


if __name__ == "__main__":
    cli()



import sys
import threading
import ctypes


class ExThread(threading.Thread):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.counter = 0

    def run(self):
        # target function of the thread class
        try:
            super().run()
        except BaseException as exception:
            print(f"Thread {self.get_id()} got {exception=}")
            print(sys.exc_info())

    def get_id(self):
        # returns id of the respective thread
        if hasattr(self, '_thread_id'):
            return self._thread_id
        for id_, thread in threading._active.items():
            if thread is self:
                return id_

    def raise_exception(self, exception):
        if not issubclass(exception, Exception):
            raise ValueError(f"Not a valid exception <{type(exception)}>.")

        thread_id = self.get_id()
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(
            thread_id, ctypes.py_object(exception)
        )
        if res > 1:
            ctypes.pythonapi.PyThreadState_SetAsyncExc(thread_id, 0)
            print('Exception raise failure')



from typing import Union, Optional

import requests


class Client:
    def __init__(
            self,
            host: str = "127.0.0.1",
            port: Union[int, str] = 8080,
    ):
        self.host = host
        self.port = port
        self.conn_str = f"http://{host}:{port}"

    def get(self, path):
        response = requests.get(url=f"{self.conn_str}/{path}")
        return response.json()

    def post(self, path, data: Optional[dict] = None, params: Optional[dict] = None):
        response = requests.post(url=f"{self.conn_str}/{path}", json=data, params=params)
        return response.json()

    def ping(self):
        return self.get(path="")

    def exit(self, force: Optional[bool] = None, wait: Optional[bool] = None) -> str:
        params = {}
        if force is not None:
            params.update(dict(force=force))
        if wait is not None:
            params.update(dict(wait=force))
        try:
            self.post(path="exit", params=params)
            return "server is still up"
        except requests.exceptions.ConnectionError:
            return "server was stopped successfully"

    def stop_all(self, force: Optional[bool] = None, wait: Optional[bool] = None):
        params = {}
        if force is not None:
            params.update(dict(force=force))
        if wait is not None:
            params.update(dict(wait=force))

        self.post(path="stop_all", params=params)

    def kill_service(self) -> str:
        try:
            self.post(path="kill_service")
            return "server is still up"
        except requests.exceptions.ConnectionError:
            return "server was stopped successfully"

    def start_worker(self):
        return self.post(path="start_worker")

    def stop_worker(self, uid: str):
        return self.post(path=f"stop_worker/{uid}")



from typing import Optional

from service_sdk.worker.worker import Worker


class Registry:
    def __init__(self, max_workers: int = 1):
        self._max_workers = max_workers
        self._workers_count = 0
        self._registry = {}

    @property
    def max(self):
        return self._max_workers

    @property
    def counter(self):
        return self._workers_count

    def add(self, worker: Worker):
        self._registry[worker.uid] = worker
        self._workers_count += 1

    def remove(self, id_: str):
        del self._registry[id_]
        self._workers_count -= 1

    def all(self):
        return list(self._registry.values())

    def get(self, id_: str) -> Optional[Worker]:
        return self._registry.get(id_)



import os
from logging import Logger
from typing import Optional, Callable, Any

import uvicorn
from fastapi import FastAPI, APIRouter, HTTPException
from starlette.routing import Route

from service_sdk.utils import force_kill_process, setup_logger
from service_sdk.manager.work_manager import WorkManager


DEFAULT_SERVICE_HOST = "127.0.0.1"
DEFAULT_SERVICE_PORT = "8080"


class Service:
    def __init__(
            self,
            host: str = DEFAULT_SERVICE_HOST,
            port: str = DEFAULT_SERVICE_PORT,
            worker_callback: Optional[Callable[[], Any]] = None,
            api: Optional[FastAPI] = None,
            manager: Optional[WorkManager] = None,
            logger: Optional[Logger] = None
    ):
        self.logger = logger or setup_logger()
        self.host = host
        self.port = int(port)
        self.api = api or FastAPI()
        self.manager = manager or WorkManager(work_callback=worker_callback)

        router = APIRouter()
        router.add_api_route(path="/", endpoint=self.root, methods=["GET"])
        router.add_api_route(path="/exit", endpoint=self.exit, methods=["POST"])
        router.add_api_route(path="/kill_service", endpoint=self.kill_service, methods=["POST"])
        router.add_api_route(path="/stop_all", endpoint=self.stop_all_worker, methods=["POST"])
        router.add_api_route(path="/start_worker", endpoint=self.start_worker, methods=["POST"])
        router.add_api_route(path="/stop_worker/{uid}", endpoint=self.stop_worker, methods=["POST"])
        self.api.include_router(router=router)

    def run(self, start_worker: bool = True):
        if start_worker:
            self.manager.start_worker()
        try:
            uvicorn.run(self.api, host=self.host, port=self.port, log_level="info")
        finally:
            self.exit()

    def exit(self, force: bool = False, wait: bool = True):
        try:
            self.stop_all_worker(force=force, wait=wait)
        finally:
            self.kill_service()

    def stop_all_worker(self, force: bool = False, wait: bool = True):
        if force:
            self.manager.kill_all(wait=wait)
        else:
            self.manager.stop_all(wait=wait)

    @staticmethod
    def kill_service():
        force_kill_process(os.getpid())

    def start_worker(self):
        self.manager.start_worker()
        return {"message": "worker started"}

    def stop_worker(self, uid: str):
        result = self.manager.stop_by_uid(uid=uid)
        if result is None:
            raise HTTPException(
                status_code=404,
                detail="Worker not found",
                headers={"X-Error": f"Worker {uid=} is not exist."},
            )
        return {"message": "worker stopped", "uid": result}

    def root(self):
        return {
            "name": self.manager.name,
            "status": self.manager.get_status(),
            "paths": [route.path for route in self.api.routes if isinstance(route, Route)]
        }
